function h = bandwidth_estimate(RT)
% Description: computes the optimal bin width.  This routine is called by RT_censoring.m
% 
% Syntax: h = bandwidth_estimate (RT)
% 
% Input: RT  a column vector containing measured RT samples (msec)
%               
% Output: h  optimal bin width


try
    h = hldpi(RT,'gauss',[],2^11);    % It is the Wand and Jones technique for bandwidth estimation 
catch    
    [~,~,h] = ksdensity(RT);  % "Density estimation." Silverman (1986) [Eqs. 3.29 and 3.30]
end

end

%% --------------------------------------------------------------------- %%
function h=hldpi(A,kernel,L,inc)
%HLDPI L-stage Direct Plug-In estimate of smoothing parameter.
%
% CALL: hs = hldpi(data,kernel,L)
%
%       hs = one dimensional value for smoothing parameter
%            given the data and kernel.  size 1 x D
%   data   = data matrix, size N x D (D = # dimensions )
%   kernel = 'epanechnikov'  - Epanechnikov kernel.
%            'biweight'      - Bi-weight kernel.
%            'triweight'     - Tri-weight kernel.
%            'triangluar'    - Triangular kernel.
%            'gaussian'      - Gaussian kernel
%            'rectangular'   - Rectanguler kernel.
%            'laplace'       - Laplace kernel.
%            'logistic'      - Logistic kernel.
%        L = 0,1,2,3,...   (default 2)
%
%  Note that only the first 4 letters of the kernel name is needed.
%
%  Example:
%   x  = wnormrnd(0,1,50,1);
%   hs = hldpi(x,'gauss',1);
%
% See also  hste, hbcv, hboot, hos, hlscv, hscv, hstt, kde, kdefun

%  Wand,M.P. and Jones, M.C. (1995)
% 'Kernel smoothing'
%  Chapman and Hall, pp 67--74

error(nargchk(1,4,nargin))
if nargin<2|isempty(kernel)
    kernel='gauss';
end
if nargin<3|isempty(L),
    L=2;
else
    L=abs(L);
end;
if nargin<4|isempty(inc)
    inc=128;
end

nfft = inc*2;
[n, d] = size(A);
if (n==1) & (d>1),
    A=A.';
    n=d;
    d=1;
end

xmin = min(A);    % Find the minimum value of A.
xmax = max(A);    % Find the maximum value of A.
xrange = xmax-xmin; % Find the range of A.

sigmaA = std(A);
iqr = abs(diff(qlevels2(A,[75 25]),1,1)); % interquartile range
k = find(iqr>0);
if any(k)
    sigmaA(k) = min(sigmaA(k), iqr(k)/1.349);
end

% R= int(mkernel(x)^2)
% mu2= int(x^2*mkernel(x))
[mu2,R] = kernelstats(kernel);

STEconstant = R /(mu2^(2)*n);

% xa holds the x 'axis' vector, defining a grid of x values where
% the k.d. function will be evaluated and plotted.

ax1 = xmin-xrange/8;
bx1 = xmax+xrange/8;

kernel2 = 'gaus';
[mu2,R] = kernelstats(kernel2); % Bug fix: Do not delete this: pab aug2005

for dim=1:d
    s = sigmaA(dim);
    ax = ax1(dim);
    bx = bx1(dim);
    
    xa = linspace(ax,bx,inc).';
    xn = linspace(0,bx-ax,inc);
    
    c = gridcount(A(:,dim),xa);
    
    %c=zeros(inc,1);
    %deltax =(bx-ax)/(inc-1);
    %xn     = (0:(inc-1))*deltax;
    %binx   = floor((A(:,dim)-ax)/deltax)+1;
    
    
    % Obtain the grid counts.
    %c = full(sparse(binx,1,(xa(binx+1)-A(:,dim)),inc,1)+...
    %         sparse(binx+1,1,(A(:,dim)-xa(binx)),inc,1))/deltax;
    
    r   = 2*L+4;
    rd2 = L+2;
    
    % Eq. 3.7 in Wand and Jones (1995)
    PSI_r =  (-1)^(rd2)*prod(rd2+1:r)/(sqrt(pi)*(2*s)^(r+1));
    PSI   = PSI_r;
    if L>0
        % High order derivatives of the Gaussian kernel
        [Kd{1:L}] = deriv(0,kernel2);
        
        % L-stage iterations to estimate PSI_4
        for ix = L:-1:1
            gi = (-2*Kd{ix}/(mu2*PSI*n))^(1/(2*ix+5));
            
            % Obtain the kernel weights.
            [KW0{1:ix}] = deriv(xn/gi,kernel2);
            kw=[KW0{ix},0,KW0{ix}([inc:-1:2])].'; % Apply 'fftshift' to kw.
            
            % Perform the convolution.
            z=real(ifft(fft(c,nfft).*fft(kw)));
            
            PSI = sum(c.*z(1:inc))/(n^2*gi^(2*ix+3));
        end
    end
    h(dim)=(STEconstant/PSI)^(1/5);
end

end

%% --------------------------------------------------------------------- %%
function [q, p]=qlevels2(r,p,method)
%QLEVELS2 Calculates quantile levels which encloses P% of data
%
%  CALL: [ql PL] = qlevels2(data,PL,method);
%
%    ql   = the discrete quantile levels, size Np x D
%    data = data matrix, size N x D (D = # of dimensions)
%    PL   = percent level vector, length Np (default [10:20:90 95 99 99.9])
%  method = 1 Interpolation so that F(X_(k)) == (k-0.5)/n. (default)
% 	    2 Interpolation so that F(X_(k)) == k/(n+1).
% 	    3 Based on the empirical distribution.
%
% QLEVELS2 sort the columns of data in ascending order and find the
%          quantile levels for each column which encloses  P% of the data.
%
% Examples : % Finding quantile levels enclosing P% of data:
%   xs  = wnormrnd(0,1,100000,1);
%   qls = qlevels2(wnormpdf(xs),[10:20:90 95 99 99.9]);
%            % compared with the exact values
%   ql  = wnormpdf(wnorminv((100-[10:20:90 95 99 99.9])/200));
%
% Finding the median of xs:
%   ql  = qlevels2(xs,50);
%
% See also  qlevels

%tested on: matlab 5.3
% History:
% revised pab
%  -fixed a bug for D>=3
%  -added different methods from wquantile
%  -updated example
% by pab 25.09.1999
%

[n, d]=size(r);
if nargin<2|isempty(p)
    p = [10:20:90 95 99 99.9];
elseif p<0 | 100<p,
    error('PL must satisfy 0 <= PL <= 100')
end
if nargin<3|isempty(method), method=1; end

if (n==1) & (d>1)
    r=r(:);
    n=d;
    d=1;
end
if d>1
    if min(size(p)) > 1
        error('Not both matrix r and matrix p input')
    end
    q = zeros(length(p),d);
else
    q = zeros(size(p));
end
p = 1-p(:)/100;
x = sort(r);


if method == 3
    qq1 = x(ceil(max(1,p*n)),:);
    qq2 = x(floor(min(p*n+1,n)),:);
    qq = (qq1+qq2)/2;
else
    x = [x(1,:); x; x(n,:)];
    if method == 2
        % This method is from Hjort's "Computer
        % intensive statistical methods" page 102
        i = p*(n+1)+1;
    else % Metod 1
        i = p*n+1.5;
    end
    iu = ceil(i);
    il = floor(i);
    d1 = (i-il)*ones(1,d);
    qq = x(il,:).*(1-d1)+x(iu,:).*d1;
end

q(:) = qq;

end

%% --------------------------------------------------------------------- %%
function [mu2, R, Rdd] = kernelstats(kernel)
%KERNELSTATS Return 2'nd order moment of kernel pdf
%            as well as the integral of the squared kernel
%            and integral of squared double derivative of kernel.
%
%  CALL:  [mu2, R, Rdd] = kernelstats(kernel)
%
% mu2    = 2'nd order moment, i.e.,int(x^2*kernel(x))
% R      = integral of squared kernel, i.e., int(kernel(x)^2)
% Rdd    = integral of squared double derivative of kernel, i.e.,
%          int( (kernel''(x))^2 ).
% kernel = string identifying the kernel, i.e., one of:
%            'epanechnikov'  - Epanechnikov kernel.
%            'biweight'      - Bi-weight kernel.
%            'triweight'     - Tri-weight kernel.
%            'triangluar'    - Triangular kernel.
%            'gaussian'      - Gaussian kernel
%            'rectangular'   - Rectanguler kernel.
%            'laplace'       - Laplace kernel.
%            'logistic'      - Logistic kernel.
%
%  Note that only the first 4 letters of the kernel name is needed.
%
% Example
%  [mu2,R]=kernelstats('triweight')
%
% See also  mkernel

% Reference
%  Wand,M.P. and Jones, M.C. (1995)
% 'Kernel smoothing'
%  Chapman and Hall, pp 176.

%History
% by pab Dec2003

switch lower(kernel(1:4))
    case 'biwe', % Bi-weight kernel
        mu2 = 1/7;
        R   = 5/7;
        Rdd = 45/2;
    case {'epan' 'epa1'}, % Epanechnikov kernel
        mu2 = 1/5;
        R   = 3/5;
        Rdd = inf;
    case {'gaus','norm'}, % Gaussian kernel
        mu2 = 1;
        R   = 1/(2*sqrt(pi));
        Rdd = 3/(8*sqrt(pi));
    case  'lapl', % Laplace
        mu2 = 2;
        R   = 1/4;
        Rdd = inf;
    case 'logi', % Logistic
        mu2 = pi^2/3;
        R=1/6;
        Rdd = 1/42;
    case {'rect','unif'}, % Rectangular
        mu2 = 1/3;
        R   = 1/2;
        Rdd = inf;
    case 'tria', % Triangular
        mu2 = 1/6;
        R   = 2/3;
        Rdd = inf;
    case 'triw', % Triweight
        mu2 = 1/9;
        R   = 350/429;
        Rdd = inf;
    otherwise
        error('Unknown kernel.')
end;

end

%% --------------------------------------------------------------------- %%
function c = gridcount(data,X)
%GRIDCOUNT D-dimensional histogram using linear binning.
%
% CALL  c = gridcount(data,X)
%
% c    = gridcount, size N x 1           if D = 1
%                   size N x N x ... x N if D > 1
% data = row vectors with D-dimensional data, size Nd x D
% X    = column vectors defining discretization, size N x D
%        Must include the range of the data.
% GRIDCOUNT obtains the grid counts using linear binning.
% There are 2 strategies: simple- or linear- binning.
% Suppose that an observation occurs at x and that the nearest point
% below and above is y and z, respectively. Then simple binning strategy
% assigns a unit weight to either y or z, whichever is closer. Linear
% binning, on the other hand, assigns the grid point at y with the weight
% of (z-x)/(z-y) and the gridpoint at z a weight of (y-x)/(z-y).
%
% In terms of approximation error of using gridcounts as pdf-estimate,
% linear binning is significantly more accurate than simple binning.
%
% NOTE: The interval [min(X);max(X)] must include the range of the data.
%       The order of C is permuted in the same order as
%       meshgrid for D==2 or D==3.
%
% Example
%  N     = 500;
%  data  = wraylrnd(1,N,1);
%  x = linspace(0,max(data)+1,50).';
%  dx = x(2)-x(1);
%  c = gridcount(data,x);
%  plot(x,c,'.')   % 1D histogram
%  plot(x,c/dx/N)  % 1D probability density plot
%  trapz(x,c/dx/N)
%
% See also  bincount, kdebin

%Reference
%  Wand,M.P. and Jones, M.C. (1995)
% 'Kernel smoothing'
%  Chapman and Hall, pp 182-192

%  History
% revised pab Feb2005
% -fixed a bug for d>2;
% by pab Dec2003

error(nargchk(2,2,nargin))

[n,d]    = size(data);
[inc,d1] = size(X);

if d~=d1
    error('Dimension 2 of data and X do not match.')
end

dx  = diff(X(1:2,:),1);
xlo = X(1,  :);
xup = X(end,:);

if any(min(data,[],1)<xlo) | any(xup<max(data,[],1))
    error('X does not include whole range of the data!')
end

if d>1
    csiz = inc(:,ones(1,d));
    n1   = n;
else
    csiz = [inc,1];
    n1   = 1;
end

binx = floor((data-xlo(ones(n1,1),:))./dx(ones(n1,1),:))+1;
w    = prod(dx);
if  d==1,
    try,
        % binc is much faster than sparse for 1D data
        % However, for N-Dimensional data sparse is sometimes faster.
        c = zeros(csiz);
        [len,bin,val] = bincount(binx,(X(binx+1)-data));
        c(bin) = val;
        [len,bin,val] = bincount(binx+1,(data-X(binx)));
        c(bin) = c(bin)+val;
        c = c/w;
    catch
        c = full(sparse(binx,1,(X(binx+1)-data),inc,1)+...
            sparse(binx+1,1,(data-X(binx)),inc,1))/w;
    end
elseif d==2
    b2 = binx(:,2);
    b1 = binx(:,1);
    c = full(sparse(b1,b2 ,abs(prod(([X(b1+1,1) X(b2+1,2)]-data),2)),inc,inc));
    c = c + sparse(b1+1,b2  ,abs(prod(([X(b1,1) X(b2+1,2)]-data),2)),inc,inc);
    c = c + sparse(b1  ,b2+1,abs(prod(([X(b1+1,1) X(b2,2)]-data),2)),inc,inc);
    c = (c + sparse(b1+1,b2+1,abs(prod(([X(b1,1) X(b2,2)]-data),2)),inc,inc))/w;
else % d>2
    useSparse = 1;
    Nc = prod(csiz);
    c  = zeros(Nc,1);
    
    fact2 = [0 inc*(1:d-1)];
    fact1 = [1 cumprod(csiz(1:d-1))];
    fact1 = fact1(ones(n,1),:);
    for ir=0:2^(d-1)-1,
        bt0 = bitget(ir,1:d);
        bt1 = ~bt0;
        
        % Convert to linear index (faster than sub2ind)
        b1  = sum((binx + bt0(ones(n,1),:)-1).*fact1,2)+1; %linear index to c
        bt2 = bt1 + fact2;
        b2  = binx + bt2(ones(n,1),:);                     % linear index to X
        if useSparse
            % Fast gridding using sparse
            c = c + sparse(b1,1,abs(prod(X(b2)-data,2)),Nc,1);
        else
            [len,bin,val] = bincount(b1,abs(prod(X(b2)-data,2)));
            c(bin)        = c(bin)+val;
        end
        
        % Convert to linear index (faster than sub2ind)
        b1  = sum((binx + bt1(ones(n,1),:)-1).*fact1,2)+1; % linear index to c
        bt2 = bt0 + fact2;
        b2  = binx + bt2(ones(n,1),:);                     % linear index to X
        
        if useSparse
            % Fast gridding using sparse
            c = c + sparse(b1,1,abs(prod(X(b2)-data,2)),Nc,1);
        else
            [len,bin,val] = bincount(b1,abs(prod(X(b2)-data,2)));
            c(bin)        = c(bin)+val;
        end
    end
    c = reshape(c/w,csiz);
end
switch d % make sure c is stored in the same way as meshgrid
    case 2,  c = c.';
    case 3,  c = permute(c,[2 1 3]);
end

end

%% --------------------------------------------------------------------- %%
function [varargout]=deriv(t,k)
%DERIV  4th, 6th, 8th and 10th derivatives of the kernel function.
%
% CALL:  [y4,y6,y8,y10] = deriv(t,kernel)
%
%   kernel = 'gaussian'      - Gaussian kernel (Currently the only
%                              supported kernel)
%
%   DERIV(T,K) finds the derivatives of the kernel K
%              at the point T.
%
% Example: [y4,y6,y8,y10]=deriv(0:3);

%tested on: matlab 5.3
%History
%revised pab Aug2005
% made it general for derivatives of any order
%revised pab dec2003
% added todo
%revised pab 16.10.1999
%  updated to matlab 5.x + documentation
% by             Christian C. Beardah 1995


% TODO % Add support for other kernels than the Gaussian

if nargin<2|isempty(k)
    k='gauss';
end

switch lower(k(1:4))
    case {'gaus'},
        phi0 = exp(-0.5*t.^2)/sqrt(2*pi);
        if 1
            % New call by pab Aug 2005
            p4 = [1 0 -6 0 +3];
            varargout{1} = polyval(p4,t).*phi0;
            
            pn = p4;
            for ix = 2:nargout
                pnp1 = polyadd([-pn,0],polyder(pn));
                pnp2 = polyadd([-pnp1,0],polyder(pnp1));
                varargout{ix} = polyval(pnp2,t).*phi0;
                pn = pnp2;
            end
            
        else % old call kept just in case
            y4=(t.^4-6*t.^2+3).*phi0;
            if nargout>1,
                y6=(t.^6-15*t.^4+45*t.^2-15).*phi0;
            end
            if nargout>2;
                y8=(t.^8-28*t.^6+210*t.^4-420*t.^2+105).*phi0;
            end;
            if nargout>3,
                y10=(t.^10-45*t.^8+630*t.^6-3150*t.^4+4725*t.^2-945).*phi0;
            end;
        end
    otherwise,
        disp('Kernel not suported')
        y4=[];y6=[];y8=[];y10=[];
end;

end

%% --------------------------------------------------------------------- %%

function r = polyadd(p, q)
%POLYADD Add polynomials.
%
%   R = POLYADD(P, Q) adds the polynomials whose coefficients are the
%   elements of the vectors P and Q.
%
%   See also POLYADD, POLYSUB, POLYMUL, POLYDIV, POLYPOW.

%   Author:      Peter J. Acklam
%   Time-stamp:  1998-06-22 20:36:21
%   E-mail:      jacklam@math.uio.no
%   WWW URL:     http://www.math.uio.no/~jacklam

error(nargchk(2, 2, nargin));

m = length(p);
n = length(q);
l = max(m, n);

r = zeros(1, l);                     % Initialize output.
r(l-m+1:l) = p;                      % Insert first polynomial.
r(l-n+1:l) = r(l-n+1:l) + q;         % Add second polynomial

r = polytrim(r);                     % Trim the result.

end

%% --------------------------------------------------------------------- %%
function r = polytrim(p)
%POLYTRIM Trim polynomial by stripping off leading zeros.
%
%   R = POLYTRIM(P) trims the polynomial P by stripping off unnecessary
%   leading zeros.
%
%   P is a vector of coefficients in decreasing order.

%   Author:      Peter J. Acklam
%   Time-stamp:  1998-06-22 20:31:39
%   E-mail:      jacklam@math.uio.no
%   WWW URL:     http://www.math.uio.no/~jacklam

k = find(p);         % Find non-zero coefficients.
if isempty(k)        % If non were found...
    r = 0;            % ...return zero polynomial...
else                 % or else...
    k = min(k);       % ...get index of first...
    n = length(p);    % ...get length of vector...
    r = p(k:n);       % ...and assign output polynomial.
end

end

%% --------------------------------------------------------------------- %%
